#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <chrono>

#define BLOCK_SIZE 1024

struct Body {
    float x=0, y = 0, z = 0, vx = 0, vy = 0, vz = 0;
};

void randBodies(Body* data, int n) {
    for (int i = 0; i < n; i++) {
        (data + i)->x = 2.0f * (rand() / (float)RAND_MAX) - 1.0f;
        (data + i)->y = 2.0f * (rand() / (float)RAND_MAX) - 1.0f;
        (data + i)->z = 2.0f * (rand() / (float)RAND_MAX) - 1.0f;
        (data + i)->vx = 2.0f * (rand() / (float)RAND_MAX) - 1.0f;
        (data + i)->vy = 2.0f * (rand() / (float)RAND_MAX) - 1.0f;
        (data + i)->vz = 2.0f * (rand() / (float)RAND_MAX) - 1.0f;
    }
}

void bodyForce(Body* p, float dt, int n) { //Glavna funkcija koju provodi svaki treth
    int i = 0;
    if (i < n) {
        float rx = 0.0f;
        float ry = 0.0f;
        float rz = 0.0f;

        for (int j = 0; j < n; j++) {
            float dx = p[j].x - p[i].x;
            float dy = p[j].y - p[i].y;
            float dz = p[j].z - p[i].z;

            float distSqr = dx * dx + dy * dy + dz * dz;

            float invDistCube = 1.0f / sqrtf(distSqr);
            float invDist3 = invDistCube * invDistCube * invDistCube;

            rx += dx * invDist3;
            ry += dy * invDist3;
            rz += dz * invDist3;

        }

        p[i].vx += dt * rx;
        p[i].vy += dt * ry;
        p[i].vz += dt * rz;
    }
    printf(".");
    i++;
}

int main() {
    int nBodies = 15000;
    const float dt = 0.01f;
    Body* p;
    p = (struct Body*)malloc(nBodies * sizeof(struct Body));

    randBodies(p, nBodies);
    auto start = std::chrono::steady_clock::now();
    for (int n = 0; n < nBodies; n++) {
        int i = 0;
        if (i < nBodies) {
            float rx = 0.0f;
            float ry = 0.0f;
            float rz = 0.0f;

            for (int j = 0; j < nBodies; j++) {
                if (i != j) {
                    float dx = p[j].x - p[i].x;
                    float dy = p[j].y - p[i].y;
                    float dz = p[j].z - p[i].z;

                    float distSqr = dx * dx + dy * dy + dz * dz;

                    float invDistCube = 1.0f / sqrtf(distSqr);
                    float invDist3 = invDistCube * invDistCube * invDistCube;

                    rx += dx * invDist3;
                    ry += dy * invDist3;
                    rz += dz * invDist3;
                }

            }

            p[i].vx += dt * rx;
            p[i].vy += dt * ry;
            p[i].vz += dt * rz;
        }
        //printf("-%d",n);
        i++;

    }
    auto end = std::chrono::steady_clock::now();
    std::chrono::duration<double> elapsed_seconds = end - start;
    std::cout << "elapsed time: " << elapsed_seconds.count()*10 << "s\n";
    for (int j1 = 0; j1 < nBodies; j1++) { // Izračunavamo poziciju svakog tijela
        p[j1].x += p[j1].vx * dt;
        p[j1].y += p[j1].vy * dt;
        p[j1].z += p[j1].vz * dt;
    }

    

    delete[] p;
    
}